package de.juplo.thymeproxy;

import de.juplo.thymeleaf.JuploDialect;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import org.apache.http.impl.client.CloseableHttpClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.web.ErrorAttributes;
import org.springframework.boot.autoconfigure.web.ErrorController;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.core.Ordered;
import org.springframework.core.env.Environment;
import org.springframework.http.HttpStatus;
import org.springframework.web.servlet.config.annotation.ViewControllerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;
import org.springframework.web.servlet.handler.SimpleMappingExceptionResolver;
import org.thymeleaf.TemplateEngine;
import org.thymeleaf.resourceresolver.IResourceResolver;
import org.thymeleaf.resourceresolver.UrlResourceResolver;
import org.thymeleaf.templateresolver.TemplateResolver;


@SpringBootApplication
public class Application extends WebMvcConfigurerAdapter
{
  @Autowired
  private ServerProperties properties;


  @Bean
  public RegexUrlHandlerMapping proxiedHandlerMapping(
      ProxyHttpRequestHandler proxy,
      UrlThymeleafViewController views
      )
  {
    RegexUrlHandlerMapping mapping = new RegexUrlHandlerMapping();
    mapping.setOrder(Ordered.HIGHEST_PRECEDENCE);
    Map<String, Object> mappings = new HashMap<>();
    mappings.put("/img/.+", proxy);
    mappings.put("/css/.+", proxy);
    mappings.put("/js/.+", proxy);
    mappings.put("/fonts/.+", proxy);
    mappings.put("/.*\\.html", views);
    mapping.setUrlMap(mappings);
	return mapping;
  }

  @Bean
  public ProxyHttpRequestHandler proxyHttpRequestHandler(
      CloseableHttpClient client,
      Environment env
      )
  {
    ProxyHttpRequestHandler handler = new ProxyHttpRequestHandler();
    handler.setClient(client);
    handler.setOrigin(env.getProperty("thymeproxy.origin"));
    handler.setTtl(30000l);
    return handler;
  }

  @Bean
  public UrlThymeleafViewController urlThymeleafViewController(
      TemplateEngine engine
      )
  {
    return new UrlThymeleafViewController(engine);
  }

  @Bean
  public TemplateResolver defaultTemplateResolver(
      IResourceResolver resources,
      Environment env
      )
  {
    TemplateResolver resolver = new TemplateResolver();
    resolver.setResourceResolver(resources);
    resolver.setPrefix("http://localhost:8080/thymeleaf/");
    resolver.setSuffix(".html");
    resolver.setTemplateMode("HTML5");
    resolver.setCharacterEncoding("UTF-8");
    resolver.setCacheable(Boolean.valueOf(env.getProperty("thymeproxy.cacheable")));
    return resolver;
  }

  @Bean
  public TemplateResolver jsonTemplateResolver(
      IResourceResolver resources,
      Environment env
      )
  {
    TemplateResolver resolver = new TemplateResolver();
    resolver.setResourceResolver(resources);
    resolver.setPrefix("http://localhost:8080/thymeleaf/");
    resolver.setSuffix(".json");
    resolver.setTemplateMode("JSON");
    resolver.setCharacterEncoding("UTF-8");
    resolver.setCacheable(Boolean.valueOf(env.getProperty("thymeproxy.cacheable")));
    return resolver;
  }

  @Bean
  public UrlResourceResolver thymeleafResourceResolver()
  {
    return new UrlResourceResolver();
  }

  @Bean
  public JuploDialect juploDialect()
  {
    return new JuploDialect();
  }

  @Bean
  public SimpleMappingExceptionResolver simpleMappingExceptionResolver()
  {
    SimpleMappingExceptionResolver resolver =
        new SimpleMappingExceptionResolver();

    Properties mappings = new Properties();
    mappings.setProperty("TemplateInputException", "templates/404");

    resolver.setExceptionMappings(mappings);
    resolver.setDefaultErrorView("templates/error");
    resolver.setWarnLogCategory("exception");
    return resolver;
  }

  @Bean
  public ErrorController errorController(
      ApplicationContext context,
      ErrorAttributes errorAttributes
      )
  {
    ExceptionResolverErrorController controller =
        new ExceptionResolverErrorController(
            context,
            errorAttributes,
            properties.getError()
            );
    controller.addErrorMapping(HttpStatus.NOT_FOUND, "templates/404");
    controller.setDefaultErrorView("templates/error");
    return controller;
  }


  @Override
  public void addViewControllers(ViewControllerRegistry registry)
  {
    registry.addViewController("/").setViewName("forward:index.html");
  }


  public static void main(String[] args)
  {
    SpringApplication.run(Application.class, args);
  }
}